package com.wrj.driver.server.controller;


import com.alibaba.fastjson.JSONObject;
import com.wrj.driver.server.constant.Constants;
import com.wrj.driver.server.dto.LoginUserDto;
import com.wrj.driver.server.response.BaseResponse;
import com.wrj.driver.server.service.ISysUserLogService;
import com.wrj.driver.server.service.ISysUserService;
import com.wrj.driver.server.vo.LoginUserVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.time.LocalDateTime;

/**
 * <p>
 * 用户表; 前端控制器
 * </p>
 *
 * @author Automated procedures
 * @since 2023-08-10
 */
@RestController
@Api(tags = "用户管理")
@RequestMapping("/sysUser")
@Slf4j
public class SysUserController {

    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISysUserLogService userLogService;

    @ApiOperation("获取验证码")
    @GetMapping("/code")
    public BaseResponse code(@RequestParam String phone) {
        log.info("获取验证码======>phone:{}", phone);
        return BaseResponse.success(userService.code(phone));
    }

    @ApiOperation("用户登陆")
    @PostMapping("/login")
    public BaseResponse login(@RequestBody @Valid LoginUserDto loginUserDto) {
        log.info("用户登录======>loginUserDto:{}", loginUserDto);
        LoginUserVo userVo = userService.login(loginUserDto);
        userLogService.saveLog("用户登录", 4,  loginUserDto.getDevice(), 1, userVo.getUserId(), JSONObject.toJSONString(loginUserDto), JSONObject.toJSONString(userVo), 0, LocalDateTime.now());
        return BaseResponse.success(userVo);
    }

    @ApiOperation("用户登出")
    @GetMapping("/loginOut")
    public BaseResponse loginOut(){
        log.info("用户登出======>{}", LocalDateTime.now());
        Long userId = userService.loginOut();
        userLogService.saveLog("用户登出", 5, null, 1, userId, null, null, 0, LocalDateTime.now());

        return BaseResponse.success(userId);
    }

    @ApiOperation("获取用户信息")
    @GetMapping("/queryUserMessage")
    public BaseResponse queryUserMessage() {
        log.info("获取用户信息======>");
        return BaseResponse.success(userService.queryUserMessage());
    }
}
