package com.wrj.driver.server.controller;


import com.wrj.driver.server.entity.WrjSysConfigList;
import com.wrj.driver.server.response.BaseResponse;
import com.wrj.driver.server.service.IWrjSysConfigListService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 系统配置列表; 前端控制器
 * </p>
 *
 * @author Automated procedures
 * @since 2023-08-10
 */
@Api(tags = "字典值列表配置")
@RestController
@RequestMapping("/wrjSysConfigList")
@Slf4j
public class WrjSysConfigListController {

    @Autowired
    private IWrjSysConfigListService configListService;

    /**
     * 根据configKey和carTypeId查询系统配置列表
     */
    @ApiOperation("根据configKey和driverTypeId查询系统配置列表")
    @GetMapping("/querySysConfigList")
    public BaseResponse querySysConfigList(@RequestParam("configKey") String configKey, @RequestParam("driverTypeId") Integer driverTypeId) {
        log.info("request to querySysConfigList ：{}, driverTypeId:{}", configKey,driverTypeId);
        List<WrjSysConfigList> resultList = configListService.querySysConfigList(configKey, driverTypeId);
        return BaseResponse.success(resultList);
    }

    /**
     * 通过多个configKey（以逗号分隔） 和 driverTypeId 查询多个配置列表
     */
    @ApiOperation("通过多个configKey（以逗号分隔） 和 driverTypeId 查询多个配置列表")
    @GetMapping("/querySysConfigMap")
    public BaseResponse querySysConfigMap(@RequestParam("configKeys") String configKeys, @RequestParam("driverTypeId") Integer driverTypeId) {
        log.info("request to querySysConfigMap ：{}, driverTypeId:{}", configKeys,driverTypeId);
        Map<String, Map<String, String>> map = configListService.querySysConfigMap(Arrays.asList(configKeys.split(",")), driverTypeId);
        return BaseResponse.success(map);
    }

    /**
     * 根据configKey和driverTypeId查询系统配置列表数量
     */
    @ApiOperation("根据configKey和driverTypeId查询系统配置列表数量")
    @GetMapping("/querySysConfigListNum")
    public BaseResponse querySysConfigListNum(@RequestParam("configKey") String configKey, @RequestParam("driverTypeId") Integer driverTypeId) {
        log.info("request to querySysConfigListNum ：{}, driverTypeId:{}", configKey,driverTypeId);
        Integer listNum = configListService.querySysConfigListNum(configKey, driverTypeId);
        return BaseResponse.success(listNum);
    }

}
