package com.wrj.driver.server.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.wrj.driver.server.dto.QuestionPracticeAddDto;
import com.wrj.driver.server.entity.WrjQuestion;
import com.wrj.driver.server.entity.WrjQuestionPractice;
import com.wrj.driver.server.entity.WrjUserErrorQuestion;
import com.wrj.driver.server.exception.BusinessException;
import com.wrj.driver.server.mapper.WrjQuestionMapper;
import com.wrj.driver.server.mapper.WrjQuestionPracticeMapper;
import com.wrj.driver.server.mapper.WrjUserErrorQuestionMapper;
import com.wrj.driver.server.service.IWrjQuestionPracticeService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;

/**
 * <p>
 * 用户答题练习表 服务实现类
 * </p>
 *
 * @author Automated procedures
 * @since 2025-06-29
 */
@Service
public class WrjQuestionPracticeServiceImpl extends ServiceImpl<WrjQuestionPracticeMapper, WrjQuestionPractice> implements IWrjQuestionPracticeService {

    @Autowired
    private WrjQuestionMapper questionMapper;

    @Autowired
    private WrjQuestionPracticeMapper practiceMapper;

    @Autowired
    private WrjUserErrorQuestionMapper errorQuestionMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean insertQuestionPractice(QuestionPracticeAddDto addDto) {

        WrjQuestion question = questionMapper.selectById(addDto.getQuestionId());
        if (question == null) {
            throw new BusinessException("题目不存在");
        }

        WrjQuestionPractice questionPractice = new WrjQuestionPractice();
        BeanUtil.copyProperties(addDto, questionPractice);

        // 插入练习
        questionPractice.setPracticeTime(LocalDateTime.now());
        practiceMapper.insert(questionPractice);

        // 判断答题是否正确
        boolean isTrue = StrUtil.equals(addDto.getAnswer(), question.getTrueAnswer());

        questionMapper.updateAnswerNumber(addDto.getQuestionId(),1, (isTrue ? 1 : 0));

        if (!isTrue) {
            // 判断是否在错题本里（答题错误会加入错题本，答题正确不会移出）
            WrjUserErrorQuestion userErrorQuestion = errorQuestionMapper.selectByUserIdAndQuestionId(addDto.getUserId(), addDto.getQuestionId());
            if (userErrorQuestion == null) {
                userErrorQuestion = new WrjUserErrorQuestion()
                        .setQuestionId(addDto.getQuestionId())
                        .setUserId(addDto.getUserId())
                        .setCreateTime(LocalDateTime.now());
                // 将错误的答题加入错题本中
                errorQuestionMapper.insert(userErrorQuestion);
            }
        }

        return Boolean.TRUE;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean clear(Long userId, String practiceType) {
        practiceMapper.clearPracticeRecord(userId, practiceType);

        return Boolean.TRUE;
    }
}
