package com.wrj.driver.server.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONArray;
import com.wrj.driver.server.constant.Constants;
import com.wrj.driver.server.dto.BaseQueryDto;
import com.wrj.driver.server.dto.QuestionAddDto;
import com.wrj.driver.server.dto.QuestionQueryDto;
import com.wrj.driver.server.dto.TestQuestionTypeDto;
import com.wrj.driver.server.entity.WrjQuestion;
import com.wrj.driver.server.entity.WrjSysConfig;
import com.wrj.driver.server.exception.BusinessException;
import com.wrj.driver.server.mapper.WrjQuestionMapper;
import com.wrj.driver.server.redis.RedisCache;
import com.wrj.driver.server.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.wrj.driver.server.vo.WrjTypeChapterVo;
import com.wrj.driver.server.vo.QuestionVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * <p>
 * 题库; 服务实现类
 * </p>
 *
 * @author Automated procedures
 * @since 2023-08-10
 */
@Service
@Slf4j
public class WrjQuestionServiceImpl extends ServiceImpl<WrjQuestionMapper, WrjQuestion> implements IWrjQuestionService {

    @Autowired
    private IWrjSysConfigService configService;

    @Autowired
    private IWrjSysConfigListService configListService;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private IWrjTypeChapterService wrjTypeChapterService;

    @Override
    public QuestionVo queryQuestionById(QuestionQueryDto queryDto) {
        QuestionQueryDto newDto = new QuestionQueryDto()
                .setQuestionId(queryDto.getQuestionId());
        List<QuestionVo> questionVos = queryQuestion(newDto);
        return CollectionUtil.isEmpty(questionVos) ? null : questionVos.get(0);
    }

    @Override
    public List<QuestionVo> queryQuestionByIdList(QuestionQueryDto queryDto) {

        QuestionQueryDto newDto = new QuestionQueryDto()
                .setQuestionIdList(queryDto.getQuestionIdList());
        return queryQuestion(newDto);
    }

    @Override
    public List<QuestionVo> queryQuestion(QuestionQueryDto queryDto) {

        List<QuestionVo> questionVos = this.getBaseMapper().queryQuestion(queryDto);
        questionOptionHandler(questionVos);
        return questionVos;
    }

    @Override
    public List<Long> queryQuestionId(QuestionQueryDto queryDto) {

        List<Long> questionIds = this.getBaseMapper().queryQuestionId(queryDto);
        return questionIds;
    }

    @Override
    public List<QuestionVo> getTestQuestion(QuestionQueryDto queryDto) {

        List<QuestionVo> resultList = new ArrayList<>();

        //查询出题比例
        BaseQueryDto queryDto1 = new BaseQueryDto()
                .setDriverTypeId(queryDto.getDriverTypeId());
        List<WrjTypeChapterVo> wrjTypeChapters = wrjTypeChapterService.queryWrjTypeChapter(queryDto1);
        if (Objects.isNull(wrjTypeChapters)) {
            log.info("carChapter is not exist");
            throw new BusinessException("获取信息异常");
        }

        for (WrjTypeChapterVo wrjTypeChapterVo : wrjTypeChapters){
            //判断题
            if (Objects.nonNull(wrjTypeChapterVo.getJudgeNum()) && wrjTypeChapterVo.getJudgeNum() > 0) {
                queryDto.setType(Constants.QUESTION_TYPE_ONE)
                        .setChapter(wrjTypeChapterVo.getChapterId())
                        .setNum(wrjTypeChapterVo.getJudgeNum());
                resultList.addAll(this.getBaseMapper().queryQuestionByRandom(queryDto));
            }
        }

        for (WrjTypeChapterVo wrjTypeChapterVo : wrjTypeChapters){
            //单选
            if (Objects.nonNull(wrjTypeChapterVo.getRadioNum()) && wrjTypeChapterVo.getRadioNum() > 0) {
                queryDto.setType(Constants.QUESTION_TYPE_TWO)
                        .setChapter(wrjTypeChapterVo.getChapterId())
                        .setNum(wrjTypeChapterVo.getRadioNum());
                resultList.addAll(this.getBaseMapper().queryQuestionByRandom(queryDto));
            }
        }

        for (WrjTypeChapterVo wrjTypeChapterVo : wrjTypeChapters){
            //多选
            if (Objects.nonNull(wrjTypeChapterVo.getMultipleChoiceNum()) && wrjTypeChapterVo.getMultipleChoiceNum() > 0) {
                queryDto.setType(Constants.QUESTION_TYPE_THREE)
                        .setChapter(wrjTypeChapterVo.getChapterId())
                        .setNum(wrjTypeChapterVo.getMultipleChoiceNum());
                resultList.addAll(this.getBaseMapper().queryQuestionByRandom(queryDto));
            }
        }
        questionOptionHandler(resultList);
        return resultList;
    }

    @Override
    public List<Long> getTestQuestionId(QuestionQueryDto queryDto) {

        List<Long> resultList = new ArrayList<>();

        //查询出题比例
        BaseQueryDto queryDto1 = new BaseQueryDto()
                .setDriverTypeId(queryDto.getDriverTypeId());
        List<WrjTypeChapterVo> wrjTypeChapters = wrjTypeChapterService.queryWrjTypeChapter(queryDto1);
        if (Objects.isNull(wrjTypeChapters)) {
            log.info("carChapter is not exist");
            throw new BusinessException("获取信息异常");
        }

        for (WrjTypeChapterVo wrjTypeChapterVo : wrjTypeChapters){
            //判断题
            if (Objects.nonNull(wrjTypeChapterVo.getJudgeNum()) && wrjTypeChapterVo.getJudgeNum() > 0) {
                queryDto.setType(Constants.QUESTION_TYPE_ONE)
                        .setChapter(wrjTypeChapterVo.getChapterId())
                        .setNum(wrjTypeChapterVo.getJudgeNum());
                resultList.addAll(this.getBaseMapper().queryQuestionIdByRandom(queryDto));
            }
        }

        for (WrjTypeChapterVo wrjTypeChapterVo : wrjTypeChapters){
            //单选
            if (Objects.nonNull(wrjTypeChapterVo.getRadioNum()) && wrjTypeChapterVo.getRadioNum() > 0) {
                queryDto.setType(Constants.QUESTION_TYPE_TWO)
                        .setChapter(wrjTypeChapterVo.getChapterId())
                        .setNum(wrjTypeChapterVo.getRadioNum());
                resultList.addAll(this.getBaseMapper().queryQuestionIdByRandom(queryDto));
            }
        }

        for (WrjTypeChapterVo wrjTypeChapterVo : wrjTypeChapters){
            //多选
            if (Objects.nonNull(wrjTypeChapterVo.getMultipleChoiceNum()) && wrjTypeChapterVo.getMultipleChoiceNum() > 0) {
                queryDto.setType(Constants.QUESTION_TYPE_THREE)
                        .setChapter(wrjTypeChapterVo.getChapterId())
                        .setNum(wrjTypeChapterVo.getMultipleChoiceNum());
                resultList.addAll(this.getBaseMapper().queryQuestionIdByRandom(queryDto));
            }
        }
        return resultList;
    }


    @Override
    public int updateQuestion(QuestionVo questionVo) {
        return this.getBaseMapper().updateQuestion(questionVo);
    }

    @Override
    public Boolean insertQuestion(QuestionAddDto addDto) {

        WrjQuestion question = new WrjQuestion();
        BeanUtils.copyProperties(addDto,question);
        //获取该车型最大的排序
        if (Objects.isNull(question.getShowOrder())){
            Integer sort = this.getBaseMapper().queryMaxSort();
            if (Objects.isNull(sort)){
                sort = 1;
            }else {
                sort = sort + 1;
            }
            question.setShowOrder(sort);
        }
        question.setIsActive(Constants.IS_ACTIVE_TRUE);
        return this.save(question);
    }

    @Override
    public Map<String, Integer> querySpecialNum(QuestionQueryDto queryDto) {

        Map<String, Integer> resultMap = redisCache.getCacheObject(Constants.SPECIAL_NUM_PREFIX);
        if (Objects.nonNull(resultMap)) {
            return resultMap;
        }
        resultMap = this.getBaseMapper().querySpecialNum(queryDto);

        if (CollectionUtil.isEmpty(resultMap)) {
            return resultMap;
        }

        WrjSysConfig tdSysConfig = configService.queryConfigByKey(Constants.REDIS_EXPIRE_TIME, Constants.DEFAULT_TYPE_ID);
        if (Objects.isNull(tdSysConfig)) {
            log.info("config is not exist");
            throw new BusinessException("系统异常");
        }
        // 塞入redis中
        redisCache.setCacheObject(Constants.SPECIAL_NUM_PREFIX, resultMap, Long.parseLong(tdSysConfig.getConfigValue()));
        return resultMap;
    }



    /**
     * 问题选项规整
     */
    private void questionOptionHandler(List<QuestionVo> questionVos) {
        if (CollectionUtil.isNotEmpty(questionVos)) {
            for (QuestionVo questionVo : questionVos) {
                //如果选项A没有值 则为判断题
                List<QuestionVo.QuestionOption> optionList = new ArrayList<>();
                if (StrUtil.isBlank(questionVo.getChooseA()) || StrUtil.equals(questionVo.getType(), Constants.QUESTION_TYPE_ONE)) {
                    optionList.add(new QuestionVo.QuestionOption("A", "正确", "A"));
                    optionList.add(new QuestionVo.QuestionOption("B", "错误", "B"));
                } else {
                    //起码ABCD都有
                    optionList.add(new QuestionVo.QuestionOption("A", questionVo.getChooseA(), "A"));
                    optionList.add(new QuestionVo.QuestionOption("B", questionVo.getChooseB(), "B"));
                    optionList.add(new QuestionVo.QuestionOption("C", questionVo.getChooseC(), "C"));
                    optionList.add(new QuestionVo.QuestionOption("D", questionVo.getChooseD(), "D"));

                    if (StrUtil.isNotBlank(questionVo.getChooseE())) {
                        optionList.add(new QuestionVo.QuestionOption("E", questionVo.getChooseE(), "E"));
                    }
                    if (StrUtil.isNotBlank(questionVo.getChooseF())) {
                        optionList.add(new QuestionVo.QuestionOption("F", questionVo.getChooseF(), "F"));
                    }
                    if (StrUtil.isNotBlank(questionVo.getChooseG())) {
                        optionList.add(new QuestionVo.QuestionOption("G", questionVo.getChooseG(), "G"));
                    }
                }
                questionVo.setOptionList(optionList);
            }
        }
    }

    public static void main(String[] args) {
        List<TestQuestionTypeDto> testQuestionTypeDtos = new ArrayList<>();
        String jsonString = JSONArray.toJSONString(testQuestionTypeDtos);
        System.out.println(jsonString);

        List<TestQuestionTypeDto> testQuestionTypeDtos1 = JSONArray.parseArray(jsonString, TestQuestionTypeDto.class);
        System.out.println(testQuestionTypeDtos1);
    }
}
