package com.wrj.driver.server.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.wrj.driver.server.constant.Constants;
import com.wrj.driver.server.entity.WrjSysConfig;
import com.wrj.driver.server.entity.WrjSysConfigList;
import com.wrj.driver.server.exception.BusinessException;
import com.wrj.driver.server.mapper.WrjSysConfigListMapper;
import com.wrj.driver.server.redis.RedisCache;
import com.wrj.driver.server.service.IWrjSysConfigListService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.wrj.driver.server.service.IWrjSysConfigService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 系统配置列表; 服务实现类
 * </p>
 *
 * @author Automated procedures
 * @since 2023-08-10
 */
@Service
@Slf4j
public class WrjSysConfigListServiceImpl extends ServiceImpl<WrjSysConfigListMapper, WrjSysConfigList> implements IWrjSysConfigListService {

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private IWrjSysConfigService configService;

    @Override
    public List<WrjSysConfigList> querySysConfigList(String configKey, Integer driverTypeId) {

        List<WrjSysConfigList> configLists = redisCache.getCacheObject(Constants.CONFIG_LIST_PREFIX + configKey + "_" + driverTypeId);
        if (CollectionUtil.isNotEmpty(configLists)) {
            return configLists;
        }
        //redis中没有则查询数据库
        LambdaQueryWrapper<WrjSysConfigList> queryWrapper = new LambdaQueryWrapper<WrjSysConfigList>()
                .eq(WrjSysConfigList::getDriverTypeId, driverTypeId)
                .eq(WrjSysConfigList::getConfigKey, configKey)
                .eq(WrjSysConfigList::getIsActive, Constants.IS_ACTIVE_TRUE)
                .orderByAsc(WrjSysConfigList::getShowOrder);
        configLists = this.getBaseMapper().selectList(queryWrapper);

        if (CollectionUtil.isEmpty(configLists)) {
            return configLists;
        }

        //获取redis缓存时间
        WrjSysConfig tdSysConfig = configService.queryConfigByKey(Constants.REDIS_EXPIRE_TIME, Constants.DEFAULT_TYPE_ID);
        if (Objects.isNull(tdSysConfig)) {
            log.info("config is not exist");
            throw new BusinessException("系统异常");
        }
        // 塞入redis中
        redisCache.setCacheObject(Constants.CONFIG_LIST_PREFIX + configKey + "_" + driverTypeId, configLists, Long.parseLong(tdSysConfig.getConfigValue()));

        return configLists;
    }

    @Override
    public Map<String, Map<String, String>> querySysConfigMap(List<String> configKeyList, Integer driverTypeId) {

        Map<String, Map<String, String>> resultMap = new HashMap<>();
        LambdaQueryWrapper<WrjSysConfigList> queryWrapper = new LambdaQueryWrapper<WrjSysConfigList>()
                .eq(WrjSysConfigList::getDriverTypeId, driverTypeId)
                .in(WrjSysConfigList::getConfigKey, configKeyList)
                .eq(WrjSysConfigList::getIsActive, Constants.IS_ACTIVE_TRUE);

        List<WrjSysConfigList> configLists = this.getBaseMapper().selectList(queryWrapper);
        for (String configKey : configKeyList) {
            Map<String, String> collect = configLists.stream().filter(s -> StrUtil.equals(s.getConfigKey(), configKey))
                    .sorted(Comparator.comparing(WrjSysConfigList::getShowOrder))
                    .collect(Collectors.toMap(WrjSysConfigList::getConfigItemCode, WrjSysConfigList::getConfigItemName));
            resultMap.put(configKey, collect);
        }
        return resultMap;
    }

    @Override
    public Integer querySysConfigListNum(String configKey, Integer driverTypeId) {
        List<WrjSysConfigList> configLists = querySysConfigList(configKey, driverTypeId);
        return CollectionUtil.isEmpty(configLists) ? 0 : configLists.size();
    }
}
